@@ -2,22 +2,17 @@
 
 import asyncio
 import json
-import sys
 
-from ._ws_impl import (CLOSED_MESSAGE, CLOSING_MESSAGE,
-                       WebSocketError, WSMessage, WSMsgType)
-from .errors import ServerDisconnectedError
-from .helpers import create_future
-
-PY_35 = sys.version_info >= (3, 5)
-PY_352 = sys.version_info >= (3, 5, 2)
+from .client_exceptions import ClientError
+from .helpers import PY_35, PY_352, Timeout, call_later, create_future
+from .http import (WS_CLOSED_MESSAGE, WS_CLOSING_MESSAGE,
+                   WebSocketError, WSMessage, WSMsgType)
 
 
 class ClientWebSocketResponse:
 
     def __init__(self, reader, writer, protocol,
                  response, timeout, autoclose, autoping, loop, *,
-                 time_service=None,
                  receive_timeout=None, heartbeat=None):
         self._response = response
         self._conn = response.connection
@@ -25,7 +20,6 @@ class ClientWebSocketResponse:
         self._writer = writer
         self._reader = reader
         self._protocol = protocol
-        self._time_service = time_service
         self._closed = False
         self._closing = False
         self._close_code = None
@@ -35,6 +29,8 @@ class ClientWebSocketResponse:
         self._autoping = autoping
         self._heartbeat = heartbeat
         self._heartbeat_cb = None
+        if heartbeat is not None:
+            self._pong_heartbeat = heartbeat/2.0
         self._pong_response_cb = None
         self._loop = loop
         self._waiting = None
@@ -55,8 +51,8 @@ class ClientWebSocketResponse:
         self._cancel_heartbeat()
 
         if self._heartbeat is not None:
-            self._heartbeat_cb = self._time_service.call_later(
-                self._heartbeat, self._send_heartbeat)
+            self._heartbeat_cb = call_later(
+                self._send_heartbeat, self._heartbeat, self._loop)
 
     def _send_heartbeat(self):
         if self._heartbeat is not None and not self._closed:
@@ -64,8 +60,8 @@ class ClientWebSocketResponse:
 
             if self._pong_response_cb is not None:
                 self._pong_response_cb.cancel()
-            self._pong_response_cb = self._time_service.call_later(
-                self._heartbeat/2.0, self._pong_not_received)
+            self._pong_response_cb = call_later(
+                self._pong_not_received, self._pong_heartbeat, self._loop)
 
     def _pong_not_received(self):
         self._closed = True
@@ -85,6 +81,14 @@ class ClientWebSocketResponse:
     def protocol(self):
         return self._protocol
 
+    def get_extra_info(self, name, default=None):
+        """extra info from connection transport"""
+        try:
+            return self._response.connection.transport.get_extra_info(
+                name, default)
+        except:
+            return default
+
     def exception(self):
         return self._exception
 
@@ -113,7 +117,7 @@ class ClientWebSocketResponse:
         # we need to break `receive()` cycle first,
         # `close()` may be called from different task
         if self._waiting is not None and not self._closed:
-            self._reader.feed_data(CLOSING_MESSAGE, 0)
+            self._reader.feed_data(WS_CLOSING_MESSAGE, 0)
             yield from self._waiting
 
         if not self._closed:
@@ -137,8 +141,8 @@ class ClientWebSocketResponse:
 
             while True:
                 try:
-                    msg = yield from asyncio.wait_for(
-                        self._reader.read(), self._timeout, loop=self._loop)
+                    with Timeout(self._timeout, loop=self._loop):
+                        msg = yield from self._reader.read()
                 except asyncio.CancelledError:
                     self._close_code = 1006
                     self._response.close()
@@ -164,28 +168,30 @@ class ClientWebSocketResponse:
                     'Concurrent call to receive() is not allowed')
 
             if self._closed:
-                return CLOSED_MESSAGE
+                return WS_CLOSED_MESSAGE
             elif self._closing:
                 yield from self.close()
-                return CLOSED_MESSAGE
+                return WS_CLOSED_MESSAGE
 
             try:
                 try:
                     self._waiting = create_future(self._loop)
-                    with self._time_service.timeout(
-                            timeout or self._receive_timeout):
+                    with Timeout(
+                            timeout or self._receive_timeout,
+                            loop=self._loop):
                         msg = yield from self._reader.read()
-                        self._reset_heartbeat()
                 finally:
+                    self._reset_heartbeat()
                     waiter = self._waiting
                     self._waiting = None
                     waiter.set_result(True)
             except (asyncio.CancelledError, asyncio.TimeoutError):
+                self._close_code = 1006
                 raise
-            except ServerDisconnectedError:
+            except ClientError:
                 self._closed = True
                 self._close_code = 1006
-                return CLOSED_MESSAGE
+                return WS_CLOSED_MESSAGE
             except WebSocketError as exc:
                 self._close_code = exc.code
                 yield from self.close(code=exc.code)
@@ -245,6 +251,8 @@ class ClientWebSocketResponse:
         @asyncio.coroutine
         def __anext__(self):
             msg = yield from self.receive()
-            if msg.type == WSMsgType.CLOSE or msg.type == WSMsgType.CLOSED:
+            if msg.type in (WSMsgType.CLOSE,
+                            WSMsgType.CLOSING,
+                            WSMsgType.CLOSED):
                 raise StopAsyncIteration  # NOQA
             return msg
